@@ -15,73 +15,71 @@
 # You should have received a copy of the GNU General Public License along with
 # Pyskool. If not, see <http://www.gnu.org/licenses/>.
 
+"""
+Defines the :class:`Eric` class.
+"""
+
 import character
-from animatorystates import *
-from ai import Write, Jump, Hit, FireCatapult, Freeze, Catch, FireWaterPistol, DumpWaterPistol, DropStinkbomb, RideBike, FallToFloor, JumpOutOfWindow, ClimbOverSkoolGate, Kiss, ReleaseMice, SKOOL_GATE
+from animatorystates import ARM_UP, BENDING_OVER, KNOCKED_OUT, RIDING_BIKE0, RIDING_BIKE1, WALK0
+from ai import Write, Jump, Hit, FireCatapult, Freeze, Catch, FireWaterPistol, DumpWaterPistol, DropStinkbomb, RideBike, FallToFloor, JumpOutOfWindow, ClimbOverSkoolGate, Kiss, ReleaseMice, SKOOL_GATE, ASSEMBLY_DIRECTION
 import lines
 import items
 import bike
 import keys
+import sound
 
-# Minimum delay between two non-immediate lines-givings by the same teacher
+#: Minimum delay between two non-immediate lines-givings by the same teacher.
 LINES_DELAY = 60
 
-# The command list Mr Wacker should use to expel Eric after he's jumped out of
-# the top-floor window
+#: The command list Mr Wacker should use to expel Eric after he's jumped out of
+#: the top-floor window.
 NOT_A_BIRD = 'ExpelEric:NotABird'
 
-# ID of the science lab storeroom door
+#: ID of the science lab storeroom door.
 STOREROOM_DOOR = 'ScienceLabDoor'
 
-# ID of the assembly hall
+#: ID of the assembly hall.
 ASSEMBLY_HALL = 'AssemblyHall'
 
-# Delay before Eric returns to the floor after jumping
+#: Delay before Eric returns to the floor after jumping.
 JUMP_DELAY = 4
-# Delay before Eric stands up after bending over
+#: Delay before Eric stands up after bending over.
 BEND_OVER_DELAY = 4
 
-# Maximum number of mice to release per attempt
+#: Maximum number of mice to release per attempt.
 MAX_MICE_RELEASE = 5
 
-# Points scored when bike is unchained
+#: Points scored when the bike is unchained.
 BIKE_COMBO_SCORE = 1000
-# Points scored when storeroom combination is obtained
+#: Points scored when the storeroom key is obtained.
 STOREROOM_COMBO_SCORE = 1000
-# Points scored when safe key is obtained
+#: Points scored when the safe key is obtained.
 SAFE_KEY_SCORE = 2000
 
-# Number of lines kissee does for Eric on each kiss
+#: Number of lines a kissee does for Eric on each kiss.
 KISS_LINES = 1000
 
 class Eric(character.Character):
+    """This class represents our hero.
+
+    :param character_id: Unique ID.
+    :param name: Display name.
+    :param flags: Character flags.
+    """
     def __init__(self, character_id, name, flags):
         character.Character.__init__(self, character_id, name, flags)
+        self.keyboard = None
         self._reinitialise()
 
-    def reinitialise(self):
-        character.Character.reinitialise(self)
-        self._reinitialise()
+    def move(self):
+        """Move Eric. This entails checking the keyboard for relevant
+        keypresses and moving Eric accordingly, and dealing with Eric if he's
+        currently engaged in an action such as firing the catapult or water
+        pistol (during which keypresses are ignored).
 
-    def _reinitialise(self):
-        self.writing = False
-        self.frozen = False
-        self.understood = False
-        self.controller = None
-        self.lines_delay = LINES_DELAY
-        self.last_lines_giver = None
-        self.inventory = set()
-        self.mice = 0
-        self.last_bike_key = None
-        self.bike_key = None
-        self.on_bike = False
-        self.started_pedalling = False
-        self.sitting_on_saddle = False
-        self.expelled = False
-        self.hide_coords = None
-
-    def move(self, keyboard):
-        self.keyboard = keyboard
+        :return: -1 if the screen should scroll right, 1 if it should scroll
+                 left, or 0 if it should not scroll (after moving Eric).
+        """
         self.get_lines()
         self.walk_delay -= 1
         if self.walk_delay > 0:
@@ -105,12 +103,12 @@ class Eric(character.Character):
                 if self.is_sitting_on_chair():
                     self.chair().vacate()
                 self.animatory_state = WALK0
-                self.beeper.make_sitting_sound()
+                self.make_sitting_sound()
             elif self.is_sitting_on_chair() and self.keyboard.was_pressed(*keys.OPEN_DESK):
                 self.open_desk(self.skool.desk(self))
             return 0
         if self.keyboard.was_pressed(*keys.SIT_STAND) and self.sit():
-            self.beeper.make_sitting_sound()
+            self.make_sitting_sound()
             return 0
         if self.keyboard.was_pressed(*keys.FIRE_CATAPULT) and self.can_fire_catapult():
             self.controller = FireCatapult()
@@ -142,8 +140,9 @@ class Eric(character.Character):
             self.controller = Catch()
             return 0
         if self.keyboard.was_pressed(*keys.MOUNT_BIKE) and self.can_mount_bike():
+            self.bike = self.cast.bike
             self.mount_bike()
-            self.controller = RideBike(self.cast.bike)
+            self.controller = RideBike(self.bike)
             return 0
         if self.keyboard.was_pressed(*keys.KISS) and self.can_kiss():
             self.previous_as = self.animatory_state
@@ -157,6 +156,7 @@ class Eric(character.Character):
         if not self.is_supported():
             self.y += 1
             return 0
+        self.base_location = None
         old_ws, old_as, old_direction = self.get_walk_state_index(), self.animatory_state, self.direction
         plant = self.cast.plant(self)
         if self.keyboard.pressed(*keys.LEFT):
@@ -185,14 +185,29 @@ class Eric(character.Character):
         return 0
 
     def can_fire_catapult(self):
+        """Return whether Eric can fire a catapult. This will be `True` if and
+        only if :meth:`~character.Character.can_fire_catapult` (on
+        :class:`~character.Character`) returns `True` and the conker is not
+        falling from the tree.
+        """
         return character.Character.can_fire_catapult(self) and not self.cast.conker_falling()
 
     def can_sit_on_stairs(self):
-        """Return whether Eric can sit on the stairs."""
+        """Return whether Eric can sit on the stairs. This will be `True` if
+        and only if the relevant lines message is defined.
+        """
         return self.cast.can_get_lines(lines.NO_SITTING_ON_STAIRS)
 
     def is_supported(self):
-        """Return whether Eric is standing on something that prevents him from falling."""
+        """Return whether Eric is standing on something that prevents him from
+        falling. The things that can prevent Eric from falling are:
+
+        * a floor
+        * a staircase
+        * a knocked out kid
+        * a plant pot
+        * a plant
+        """
         return (self.staircase
             or self.cast.is_standing_on_kid(self)
             or self.skool.on_floor(self)
@@ -200,15 +215,28 @@ class Eric(character.Character):
             or self.cast.plant(self))
 
     def can_open_door(self, door):
+        """Return whether Eric can open a specific door. This will be `False`
+        unless the door in question is the storeroom door and Eric has the key
+        to it.
+
+        :param door: The door to check.
+        """
         if door.barrier_id == STOREROOM_DOOR and items.STOREROOM_KEY in self.inventory:
             return True
         return character.Character.can_open_door(self, door)
 
     def open_door(self):
+        """Open the door that Eric is facing.
+
+        :return: 0 (to indicate that the screen should not scroll).
+        """
         self.skool.move_door(self.barrier.barrier_id, False)
         return 0
 
     def get_lines(self):
+        """Check whether Eric is up to anything that merits being given lines,
+        and make any nearby teacher who can see him give the lines.
+        """
         self.lines_delay = max(0, self.lines_delay - 1)
         if self.lines_delay > LINES_DELAY / 2:
             return
@@ -216,7 +244,16 @@ class Eric(character.Character):
             self.last_lines_giver = None
         room = self.skool.room(self)
         home_room = self.skool.get_home_room()
-        if self.skool.in_no_go_zone(self):
+        if self.base_location:
+            # Use Eric's pre-jump coordinates for comparison if he's jumping
+            x, y = self.base_location
+        elif self.bike and not self.sitting_on_saddle:
+            # Use the bike's y-coordinate if Eric's standing on the saddle
+            x, y = self.x, self.bike.y
+        else:
+            # Otherwise use Eric's actual location
+            x, y, = self.x, self.y
+        if self.skool.in_no_go_zone(x, y):
             self.alert_lines_givers(lines.GET_OUT, True)
         elif self.should_get_along():
             self.alert_lines_givers(lines.GET_ALONG, True)
@@ -224,7 +261,7 @@ class Eric(character.Character):
             self.alert_lines_givers(lines.NO_SITTING_ON_STAIRS, True)
         elif self.is_knocked_out():
             self.alert_lines_givers(lines.GET_UP, True)
-        elif self.on_bike and not self.skool.in_playground(self):
+        elif self.bike and not self.skool.in_playground(self):
             self.alert_lines_givers(lines.NO_BIKES, True)
         elif self.skool.plant_pot(self.x, self.y) or self.cast.plant(self):
             self.alert_lines_givers(lines.GET_OFF_PLANT, True)
@@ -233,7 +270,7 @@ class Eric(character.Character):
             if self.is_sitting_on_floor():
                 # Eric is sitting on the floor...
                 if room and room.room_id == ASSEMBLY_HALL:
-                    if self.direction < 0:
+                    if self.direction != ASSEMBLY_DIRECTION:
                         # ...facing left in the assembly hall...
                         if is_assembly:
                             # ...and it's assembly - tell him to sit facing the
@@ -261,20 +298,22 @@ class Eric(character.Character):
         """Return whether Eric is somewhere other than he should be."""
         return self.skool.should_get_along(self)
 
-    def make_walking_sound(self, index):
-        self.beeper.make_walking_sound(index)
-
     def walk(self, on_stairs=False):
+        """Move Eric one step in the direction he is facing. If Eric moves
+        close enough to the left or right edge of the screen, it will scroll.
+        """
         character.Character.walk(self, on_stairs)
         if not self.midstride():
             return self.screen.get_scroll_increment(self.x)
         return 0
 
     def dethrone(self):
+        """Push Eric out of his seat and play a suitable sound effect."""
         self.sit_on_floor()
-        self.beeper.make_knocked_out_sound()
+        self.play_sound(sound.KNOCKED_OUT, sound.ASYNC)
 
     def deck(self):
+        """Knock Eric out and play a suitable sound effect."""
         if self.is_sitting_on_chair():
             self.chair().vacate()
         self.knock_over()
@@ -282,9 +321,15 @@ class Eric(character.Character):
             self.writing = False
             self.keyboard.finish_writing()
             self.controller = None
-        self.beeper.make_knocked_out_sound()
+        self.play_sound(sound.KNOCKED_OUT, sound.ASYNC)
 
     def alert_lines_givers(self, message_id, reset_delay=False):
+        """Look for a lines-giver in the vicinity of Eric; if one is found,
+        make him give lines to Eric.
+
+        :param message_id: The ID of the lines message to scream.
+        :param reset_delay: Whether to reset the lines-giving delay.
+        """
         nearby_lines_givers = self.cast.get_nearby_lines_givers(self)
         if nearby_lines_givers:
             lines_giver = nearby_lines_givers[0]
@@ -303,21 +348,38 @@ class Eric(character.Character):
             self.last_lines_giver = lines_giver
 
     def aim_catapult(self):
+        """Make Eric aim his catapult (raise it to eye level). Also alert any
+        lines-givers in the vicinity, and play a sound effect.
+        """
         character.Character.aim_catapult(self)
         self.alert_lines_givers(lines.NO_CATAPULTS)
-        self.beeper.make_catapult_sound()
+        self.play_sound(sound.CATAPULT, sound.ASYNC)
 
     def has_water_pistol(self):
+        """Return whether Eric has a water pistol (containing either water or
+        sherry).
+        """
         return items.WATER_PISTOL in self.inventory or items.SHERRY_PISTOL in self.inventory
 
     def can_fire_water_pistol(self):
+        """Return whether Eric can fire a water pistol. This will be `True` if
+        Character.can_fire_water_pistol returns `True` and Eric actually has
+        the water pistol.
+        """
         if self.has_water_pistol():
             return character.Character.can_fire_water_pistol(self)
 
     def can_dump_water_pistol(self):
+        """Return whether Eric can throw away the water pistol. This will be
+        `True` if Eric has the water pistol and has a
+        :data:`~animatorystates.BENDING_OVER` sprite defined.
+        """
         return self.has_water_pistol() and self.can_bend_over()
 
     def dump_water_pistol(self):
+        """Make Eric throw away the water pistol. The water pistol will be
+        filled with water and placed in a desk chosen at random.
+        """
         pistol = None
         if items.WATER_PISTOL in self.inventory:
             pistol = items.WATER_PISTOL
@@ -330,18 +392,30 @@ class Eric(character.Character):
         self.walk_delay = BEND_OVER_DELAY
 
     def fire_water_pistol(self):
+        """Make Eric fire the water pistol. Also alert any lines-givers in the
+        vicinity, and play a sound effect.
+        """
         liquid = character.SHERRY if items.SHERRY_PISTOL in self.inventory else character.WATER
         character.Character.fire_water_pistol(self, liquid)
         self.alert_lines_givers(lines.NO_WATERPISTOLS)
-        self.beeper.make_catapult_sound()
+        self.play_sound(sound.WATER_PISTOL, sound.ASYNC)
 
     def can_drop_stinkbomb(self):
+        """Return whether Eric can drop a stinkbomb. This will be `True` if
+        :meth:`~character.Character.can_drop_stinkbomb` (on
+        :class:`~character.Character`) returns `True` and Eric has a stinkbomb
+        and is standing on a floor (as opposed to a staircase).
+        """
         return self.floor and self.has_stinkbomb() and character.Character.can_drop_stinkbomb(self)
 
     def has_stinkbomb(self):
+        """Return whether Eric has at least one stinkbomb."""
         return any([i in self.inventory for i in (items.STINKBOMBS1, items.STINKBOMBS2, items.STINKBOMBS3)])
 
     def drop_stinkbomb(self):
+        """Make Eric drop a stinkbomb. Also alert any lines-givers in the
+        vicinity.
+        """
         character.Character.drop_stinkbomb(self)
         if items.STINKBOMBS3 in self.inventory:
             self.inventory.remove(items.STINKBOMBS3)
@@ -355,10 +429,16 @@ class Eric(character.Character):
         self.alert_lines_givers(lines.NO_STINKBOMBS)
 
     def open_desk(self, desk):
+        """Make Eric raise a desk lid.
+
+        :param desk: The desk to open.
+        """
         if desk:
             self.cast.open_desk(self, desk)
 
     def collect_desk_contents(self, desk):
+        """Place the contents (if any) of a desk in Eric's inventory. Also play
+        a sound effect if the desk is not empty."""
         if desk.contents:
             if desk.contents == items.STINKBOMBS3:
                 for item in (items.STINKBOMBS2, items.STINKBOMBS1):
@@ -367,51 +447,89 @@ class Eric(character.Character):
             self.inventory.add(desk.contents)
             if desk.contents == items.WATER_PISTOL:
                 desk.empty()
-            self.beeper.make_bingo_sound()
+            self.play_sound(sound.DESK)
             self.print_inventory()
 
     def punch(self):
+        """Make Eric throw a punch. Also alert any lines-givers in the
+        vicinity.
+        """
         character.Character.punch(self)
         self.alert_lines_givers(lines.NO_HITTING)
 
     def jump(self):
+        """Make Eric jump. Also alert any lines-givers in the vicinity."""
+        if not self.base_location:
+            self.base_location = (self.x, self.y)
         self.previous_as = self.animatory_state
         self.y -= 1
         self.walk_delay = JUMP_DELAY
         self.animatory_state = ARM_UP
         self.alert_lines_givers(lines.NO_JUMPING)
 
+    def descend(self):
+        """Make Eric finish a jump. If Eric jumped up to the safe, or a shield,
+        or an open drinks cabinet while carrying a water pistol, appropriate
+        action is taken. Eric will descend to the floor unless the jump placed
+        him on a knocked out kid or a plant pot.
+        """
+        x, y = self.get_location_above_hand()
+        self.skool.check_safe(x, y, items.SAFE_KEY in self.inventory)
+        self.check_shields_at(x, y)
+        if self.skool.check_drinks_cabinet(x, y) and items.WATER_PISTOL in self.inventory:
+            self.inventory.remove(items.WATER_PISTOL)
+            self.inventory.add(items.SHERRY_PISTOL)
+            self.print_inventory()
+            self.play_sound(sound.SHERRY)
+        if not self.cast.is_standing_on_kid(self) and not self.skool.plant_pot(self.x, self.y):
+            self.y += 1
+            self.base_location = None
+        self.animatory_state = self.previous_as
+
     def can_bend_over(self):
+        """Return whether Eric can bend over. This will be `True` if and only
+        if he has a :data:`~animatorystates.BENDING_OVER` sprite defined.
+        """
         return BENDING_OVER in self.as_dict_L
 
     def bend_over(self):
+        """Make Eric bend over (as when catching or releasing mice)."""
         self.previous_as = self.animatory_state
         self.walk_delay = 1
         self.animatory_state = BENDING_OVER
 
     def catch_animal(self):
+        """Make Eric catch any animal that is close at hand. If Eric catches
+        something, a sound effect is played.
+        """
         animal = self.cast.get_animal(self)
         if animal:
             if animal.is_mouse():
                 self.skool.add_to_score(10)
-                self.beeper.make_mouse_sound()
+                self.play_sound(sound.MOUSE)
                 self.mice += 1
                 self.print_mouse_inventory()
                 self.cast.caught_mouse(animal)
             elif animal.is_frog():
                 animal.hide()
-                self.beeper.make_bingo_sound()
+                self.play_sound(sound.FROG)
                 self.inventory.add(items.FROG)
                 self.print_inventory()
         self.walk_delay = BEND_OVER_DELAY
 
     def print_mouse_inventory(self):
+        """Print Eric's mouse inventory."""
         self.screen.print_mice(self.mice, self.skool.get_mouse_image())
 
     def can_release_mice(self):
+        """Return whether Eric can release mice at the moment. This will be
+        `True` if Eric is standing on a floor (as opposed to a staircase), has
+        at least one mouse, and can bend over.
+        """
         return self.floor and self.mice > 0 and self.can_bend_over()
 
     def release_mice(self):
+        """Make Eric release some mice."""
         if self.mice > 0:
             num_mice = min(self.mice, MAX_MICE_RELEASE)
             self.mice -= num_mice
@@ -420,9 +538,11 @@ class Eric(character.Character):
         self.walk_delay = BEND_OVER_DELAY
 
     def print_inventory(self):
+        """Print Eric's inventory."""
         self.screen.print_inventory(self.skool.get_inventory_images(self.inventory))
 
     def stand_up(self):
+        """Make Eric stand up (after sitting down or bending over)."""
         self.animatory_state = self.previous_as
         self.walk_delay = 1
 
@@ -438,24 +558,16 @@ class Eric(character.Character):
         return location
 
     def get_location_above_hand(self):
+        """Return the location above Eric's hand. This location will be used to
+        check for the presence of a shield, the safe, or a cup.
+        """
         return (self.x + self.direction + 1, self.y)
 
-    def descend(self):
-        """Make Eric finish his jump."""
-        x, y = self.get_location_above_hand()
-        self.skool.check_safe(x, y, items.SAFE_KEY in self.inventory)
-        self.check_shields_at(x, y)
-        if self.skool.check_drinks_cabinet(x, y) and items.WATER_PISTOL in self.inventory:
-            self.inventory.remove(items.WATER_PISTOL)
-            self.inventory.add(items.SHERRY_PISTOL)
-            self.print_inventory()
-            self.beeper.make_sherry_sound()
-        if not self.cast.is_standing_on_kid(self) and not self.skool.plant_pot(self.x, self.y):
-            self.y += 1
-        self.animatory_state = self.previous_as
-
     def write(self):
-        """Control Eric when he's writing on a board. Return True if Eric has finished writing."""
+        """Control Eric when he's writing on a blackboard.
+
+        :return: `True` if Eric has finished writing, `False` otherwise.
+        """
         blackboard = self.skool.room(self).blackboard
         if self.has_arm_raised():
             self.lower_arm()
@@ -481,79 +593,181 @@ class Eric(character.Character):
             self.walk_delay = 1
 
     def check_combinations(self, blackboard):
+        """Check whether the bike or storeroom combination has been written on
+        any of the blackboards. If the bike combination has been written on a
+        blackboard, unchain the bike and play a sound effect. If the storeroom
+        combination has been written on a blackboard, give Eric the key to that
+        room and play a sound effect.
+        """
         if not self.cast.is_bike_visible() and self.skool.got_bike_combination(blackboard):
             self.skool.add_to_score(BIKE_COMBO_SCORE // 10)
             self.skool.unchain_bike()
-            self.beeper.make_bingo_sound()
+            self.play_sound(sound.BIKE)
         elif items.STOREROOM_KEY not in self.inventory and self.skool.got_storeroom_combination(blackboard):
             self.skool.add_to_score(STOREROOM_COMBO_SCORE // 10)
             self.inventory.add(items.STOREROOM_KEY)
             self.print_inventory()
-            self.beeper.make_bingo_sound()
+            self.play_sound(sound.STOREROOM_KEY)
 
     def is_eric(self):
+        """Return whether this character is Eric.
+
+        :return: `True`.
+        """
         return True
 
     def freeze(self):
+        """Attempt to freeze Eric. The attempt will fail if Eric is writing on
+        a blackboard.
+
+        :return: `True` if Eric was frozen, `False` otherwise.
+        """
         if not self.writing:
             self.controller = Freeze()
             self.frozen = True
         return self.frozen
 
     def unfreeze(self):
+        """Unfreeze Eric."""
         if self.frozen:
             self.controller = None
             self.frozen = False
             self.understood = False
 
     def check_understanding(self):
+        """Check whether Eric indicated understanding of the message just
+        delivered to him.
+        """
         if self.keyboard.was_pressed(*keys.UNDERSTOOD):
             self.understood = True
 
     def understood_message(self):
+        """Return whether Eric understood the message just delivered to him."""
         return self.understood
 
     def paralyse(self):
+        """Signal that Eric is paralysed (as after falling from the top-floor
+        window). Mr Wacker will be alerted.
+        """
         self.cast.expel_eric(NOT_A_BIRD)
 
     def take_safe_key(self):
+        """Make Eric collect the safe key. A celebratory sound effect will be
+        played.
+        """
         if items.SAFE_KEY not in self.inventory:
             self.skool.add_to_score(SAFE_KEY_SCORE // 10)
-            self.beeper.make_safe_key_sound()
+            self.play_sound(sound.SAFE_KEY, sound.SUSPEND)
             self.inventory.add(items.SAFE_KEY)
             self.print_inventory()
 
     def can_kiss(self):
+        """Return whether anyone in the cast can kiss Eric."""
         return self.cast.has_kissees()
 
     def kissee(self):
+        """Return the first kissable candidate in front of Eric, or `None` if
+        there is none.
+        """
         return self.cast.kissee()
 
     def kiss(self):
+        """Decrease Eric's lines total and play the kissing sound effect."""
         self.skool.add_lines(-KISS_LINES // 10)
-        self.beeper.make_bingo_sound()
+        self.play_sound(sound.KISS)
 
     def hide(self):
+        """Hide Eric. This is used when Eric is kissing someone, and two
+        sprites become one.
+        """
         self.hide_coords = (self.x, self.y)
         character.Character.hide(self)
 
     def unhide(self):
+        """Unhide Eric. Eric's pre-kiss coordinates are restored."""
         if self.hide_coords:
             self.x, self.y = self.hide_coords
             self.hide_coords = None
 
+    def fall_to_floor(self):
+        """Make Eric land on the floor in the sitting position, and play a
+        sound effect.
+        """
+        self.sit_on_floor()
+        self.play_sound(sound.KNOCKED_OUT, sound.ASYNC)
+
+    def fall_off_plant(self):
+        """Make Eric fall off a plant that has just died."""
+        self.controller = FallToFloor()
+
+    def fly(self, x_inc, y_inc):
+        """Move Eric to the next spot in his flight out of a window or over the
+        skool gate.
+
+        :param x_inc: The x-coordinate increment.
+        :param y_inc: The y-coordinate increment.
+        """
+        self.x += x_inc * self.direction
+        self.y += y_inc
+        self.make_walking_sound(self.x % 4)
+
+    def end_flight(self):
+        """Play a sound effect if Eric did not land on his feet after flying
+        out of a window or over the skool gate.
+        """
+        if not self.is_standing():
+            self.play_sound(sound.KNOCKED_OUT, sound.ASYNC)
+
+    #//////////////////////////////////////////////////////////////////////////
+    # Initialisation
+    #//////////////////////////////////////////////////////////////////////////
+    def reinitialise(self):
+        """Perform generic reinitialisation of Eric."""
+        character.Character.reinitialise(self)
+        self._reinitialise()
+
+    def _reinitialise(self):
+        """Perform specific reinitialisation of Eric."""
+        self.writing = False
+        self.frozen = False
+        self.understood = False
+        self.controller = None
+        self.lines_delay = LINES_DELAY
+        self.last_lines_giver = None
+        self.inventory = set()
+        self.mice = 0
+        self.last_bike_key = None
+        self.bike_key = None
+        self.bike = None
+        self.started_pedalling = False
+        self.sitting_on_saddle = False
+        self.expelled = False
+        self.hide_coords = None
+        self.base_location = None
+
+    #//////////////////////////////////////////////////////////////////////////
+    # Bike
+    #//////////////////////////////////////////////////////////////////////////
     def can_mount_bike(self):
+        """Return whether Eric can mount the bike at the moment. This will be
+        `True` if Eric has the :data:`~animatorystates.RIDING_BIKE0` sprite
+        defined and is standing next to the bike.
+        """
         return RIDING_BIKE0 in self.as_dict_L and self.cast.is_beside_bike(self)
 
     def mount_bike(self):
+        """Make Eric mount the bike."""
         self.last_bike_key = None
-        self.on_bike = True
         self.started_pedalling = False
         self.sitting_on_saddle = True
         self.previous_as = self.animatory_state
         self.animatory_state = RIDING_BIKE0
 
     def check_bike_keys(self):
+        """Check whether any key related to bike movement has been pressed. Any
+        bike key pressed is stored and acted upon later by another bike-related
+        method.
+        """
         self.walk_delay = bike.SPEED
         self.bike_key = None
         for key in keys.LEFT + keys.RIGHT + keys.UP + keys.DOWN + keys.JUMP:
@@ -562,6 +776,12 @@ class Eric(character.Character):
                 break
 
     def rode_into_barrier(self, bike, barrier_id=None):
+        """Return whether Eric rode the bike into a wall or door.
+
+        :param bike: The bike Eric is riding.
+        :param barrier_id: The ID of the wall or door to check; if `None`, all
+                           doors and walls are checked.
+        """
         if bike.is_visible():
             return bike.hit_barrier(barrier_id)
         # Check from a distance of 1 if travelling leftwards so that the bike
@@ -573,6 +793,14 @@ class Eric(character.Character):
         return barrier is not None
 
     def pedalled(self):
+        """Return whether Eric pedalled the bike since the last time he was
+        moved. This will be `True` if and only if:
+
+        * he is sitting on the saddle, and
+        * 'left' or 'right' was pressed, and
+        * the bike key pressed on the last call is different from the one
+          pressed on this call
+        """
         if self.sitting_on_saddle:
             pedalled = self.bike_key in keys.LEFT + keys.RIGHT and self.bike_key != self.last_bike_key
             if pedalled:
@@ -585,66 +813,111 @@ class Eric(character.Character):
         return False
 
     def pedal(self):
+        """Make Eric pedal the bike. The Eric-on-a-bike sprite is moved to the
+        next phase of animation.
+        """
         if self.animatory_state == RIDING_BIKE0:
             self.animatory_state = RIDING_BIKE1
         else:
             self.animatory_state = RIDING_BIKE0
 
     def move_bike(self):
+        """Move the bike (while Eric's on it)."""
         self.x += self.direction
 
     def dismounted(self):
+        """Return whether Eric dismounted from the bike. This will be `True`
+        if Eric was sitting on the saddle and 'down' was pressed.
+        """
         return self.sitting_on_saddle and self.bike_key in keys.DOWN
 
     def dismount(self):
-        self.on_bike = False
+        """Make Eric dismount from the bike."""
+        self.bike = None
 
     def stood_on_saddle(self):
+        """Return whether Eric stood on the saddle of the bike. This will be
+        `True` if Eric was sitting on the saddle and 'up' was pressed.
+        """
         if self.sitting_on_saddle and self.bike_key in keys.UP:
             self.last_bike_key = self.bike_key
             self.sitting_on_saddle = False
         return not self.sitting_on_saddle
 
     def stand_on_saddle(self):
+        """Make Eric stand on the saddle of the bike."""
         self.animatory_state = WALK0
         self.x -= self.direction
         self.y -= 1
 
     def got_back_on_saddle(self):
+        """Return whether Eric got back on the saddle of the bike. This will be
+        `True` if Eric was standing on the saddle and 'down' was pressed.
+        """
         if not self.sitting_on_saddle and self.bike_key in keys.DOWN:
             self.last_bike_key = self.bike_key
             self.sitting_on_saddle = True
         return self.sitting_on_saddle
 
     def get_back_on_saddle(self):
+        """Make Eric get back on the saddle of the bike."""
         self.animatory_state = RIDING_BIKE0
         self.x += self.direction
         self.y += 1
 
-    def jumped(self):
-        return not self.sitting_on_saddle and self.bike_key in keys.JUMP + keys.UP
+    def jumped_off_saddle(self):
+        """Return whether Eric jumped off the saddle of the bike. This will be
+        `True` if Eric was standing on the saddle and 'up' or 'jump' was
+        pressed.
+        """
+        if not self.sitting_on_saddle and self.bike_key in keys.JUMP + keys.UP:
+            self.base_location = (self.x, self.bike.y)
+            return True
+        return False
 
     def check_cup(self):
+        """Check whether Eric managed to reach a cup while in possession of the
+        frog. If so, the frog is placed in the cup and a sound effect is
+        played.
+        """
         if items.FROG in self.inventory:
             cup = self.skool.cup(*self.get_location_above_hand())
             if cup:
                 self.cast.insert_frog(cup)
                 self.inventory.remove(items.FROG)
                 self.print_inventory()
-                self.beeper.make_bingo_sound()
+                self.play_sound(sound.FROG)
 
-    def fall_to_floor(self):
-        self.sit_on_floor()
-        self.beeper.make_knocked_out_sound()
+    def landed(self):
+        """Return whether Eric has landed after jumping from the saddle of the
+        bike.
+        """
+        if self.y >= self.base_location[1] and self.get_floor():
+            self.base_location = None
+            return True
+        return False
 
-    def fall_off_plant(self):
-        self.controller = FallToFloor()
+    #//////////////////////////////////////////////////////////////////////////
+    # Sound effects
+    #//////////////////////////////////////////////////////////////////////////
+    def play_sound(self, sound_id, mode=sound.BLOCK):
+        """Play a sound effect.
 
-    def fly(self, x_inc, y_inc):
-        self.x += x_inc * self.direction
-        self.y += y_inc
-        self.make_walking_sound(self.x % 4)
+        :param sound_id: The ID of the sound effect.
+        :param mode: See :meth:`sound.Beeper.play`.
+        """
+        self.skool.play_sound(sound_id, mode)
+
+    def make_sitting_sound(self):
+        """Play a sitting sound effect. This will be a walking sound effect,
+        chosen at random from those available.
+        """
+        self.skool.make_sitting_sound()
+
+    def make_walking_sound(self, index):
+        """Make a walking sound effect.
+
+        :param index: The index of the walking sound effect.
+        """
+        self.skool.make_walking_sound(index)
 
-    def end_flight(self):
-        if not self.is_standing():
-            self.beeper.make_knocked_out_sound()
