@@ -15,12 +15,27 @@
 # You should have received a copy of the GNU General Public License along with
 # Pyskool. If not, see <http://www.gnu.org/licenses/>.
 
-SAFE_KEY = 'SAFE_KEY'
-STOREROOM_KEY = 'STOREROOM_KEY'
+"""
+IDs of the items that can be displayed in Eric's inventory. The image
+specifications for each item are defined in the `Inventory` section of the ini
+file.
+"""
+
+#: Captured frog.
 FROG = 'FROG'
-WATER_PISTOL = 'WATER_PISTOL'
-SHERRY_PISTOL = 'SHERRY_PISTOL'
-STINKBOMBS3 = 'STINKBOMBS3'
-STINKBOMBS2 = 'STINKBOMBS2'
-STINKBOMBS1 = 'STINKBOMBS1'
+#: Captured mouse.
 MOUSE = 'MOUSE'
+#: Safe key.
+SAFE_KEY = 'SAFE_KEY'
+#: Water pistol containing sherry.
+SHERRY_PISTOL = 'SHERRY_PISTOL'
+#: Three stinkbombs.
+STINKBOMBS3 = 'STINKBOMBS3'
+#: Two stinkbombs.
+STINKBOMBS2 = 'STINKBOMBS2'
+#: One stinkbomb.
+STINKBOMBS1 = 'STINKBOMBS1'
+#: Science Lab storeroom key.
+STOREROOM_KEY = 'STOREROOM_KEY'
+#: Water pistol containing water.
+WATER_PISTOL = 'WATER_PISTOL'
