@@ -15,27 +15,49 @@
 # You should have received a copy of the GNU General Public License along with
 # Pyskool. If not, see <http://www.gnu.org/licenses/>.
 
+"""
+Keep track of the score, lines total and high score.
+"""
+
 class Scoreboard:
+    """The scoreboard.
+
+    :type screen: :class:`~graphics.Screen`
+    :param screen: The screen on which the scoreboard is displayed.
+    """
     def __init__(self, screen):
         self.screen = screen
         self.hiscore = 0
-        self.reset()
+        self._reset()
 
-    def reset(self):
+    def _reset(self):
+        """Set the lines total and the score to zero."""
         self.score = self.lines = 0
 
     def add_to_score(self, addend):
+        """Add points to the score and print it.
+
+        :param addend: The number of points to add.
+        """
         self.score += addend
         self.screen.print_score(self.score)
 
     def add_lines(self, addend):
+        """Add lines to the lines total and print it.
+
+        :param addend: The number of lines to add.
+        """
         self.lines = max(self.lines + addend, 0)
         self.screen.print_lines(self.lines)
 
     def reinitialise(self):
+        """Reinitialise the scoreboard after a game has ended. The current
+        score becomes the new high score if necessary, the score and lines
+        total are reset to zero, and all three numbers are printed.
+        """
         if self.score > self.hiscore:
             self.hiscore = self.score
             self.screen.print_hi_score(self.hiscore)
-        self.reset()
+        self._reset()
         self.screen.print_score(self.score)
         self.screen.print_lines(self.lines)
